/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.message;

import java.net.URI;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.commons.log.FlowAttributes;
import org.ow2.petals.commons.log.FlowAttributesExchangeHelper;
import org.ow2.petals.component.framework.junit.impl.ServiceConfiguration;
import org.ow2.petals.component.framework.junit.impl.mock.MockServiceEndpoint;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.jbi.messaging.exchange.impl.MessageExchangeImpl;

import com.ebmwebsourcing.easycommons.uuid.SimpleUUIDGenerator;

/**
 * 
 * This is meant to be used by client code to send message to the component, either from the tests or from the external
 * services.
 * 
 * @author Marc Jambert - EBM WebSourcing
 */
public abstract class AbstractWrappedToComponentMessage extends AbstractWrappedMessage {

    private static final SimpleUUIDGenerator UUID_GENERATOR = new SimpleUUIDGenerator();

    public AbstractWrappedToComponentMessage(final MessageExchange messageExchange) {
        super(assertAndCast(messageExchange));
    }

    protected AbstractWrappedToComponentMessage(final ServiceConfiguration serviceConfiguration,
            final QName operation, final URI mep, final FlowAttributes flowAttributes) {
        super(AbstractWrappedToComponentMessage.createMessageExchange(serviceConfiguration.getEndpointName(),
                serviceConfiguration.getServiceQName(), serviceConfiguration.getInterfaceQName(),
                operation, mep, flowAttributes));
    }

    public static PetalsMessageExchange assertAndCast(final MessageExchange exchange) {
        assert exchange instanceof PetalsMessageExchange : "The exchange should be a PetalsMessageExchange: "+exchange.getClass().getName();
        return (PetalsMessageExchange) exchange;
    }

    private static final PetalsMessageExchange createMessageExchange(final String endpointName, final QName serviceName,
            final QName interfaceName, final QName operation, final URI mep, final FlowAttributes flowAttributes) {
        final ServiceEndpoint mockServiceEndpoint = new MockServiceEndpoint(endpointName, serviceName);

        // TODO we should rely on the factory to build it...
        final MessageExchangeImpl messageExchange = new MessageExchangeImpl(UUID_GENERATOR.getNewID());
        messageExchange.setInterfaceName(interfaceName);
        messageExchange.setEndpoint(mockServiceEndpoint);
        messageExchange.setService(serviceName);
        messageExchange.setOperation(operation);
        messageExchange.setPattern(mep);
        FlowAttributesExchangeHelper.setFlowAttributes(messageExchange, flowAttributes);
        return messageExchange;
    }
}
